TITLE	MungeIt.asm

.486
.model flat, stdcall		; memory model, calling convention
option casemap:none		; case sensitive


; ----------------------------------------------------------------------------
; system includes (both .inc and .lib)
; ============================================================================
include \masm32\include\windows.inc
include \masm32\include\kernel32.inc

includelib \masm32\lib\kernel32.lib


; ----------------------------------------------------------------------------
; module data (both initialized and uninitialized
; ============================================================================
.data?
align 16
	hInstance	dd ?


.const
TestFileSpecName	db 'c:\junk.txt', 0
BUFFER_SIZE			equ 4000h

.code

; ----------------------------------------------------------------------------
; prototype:
;		void __stdcall EncryptBuffer(char* Buffer, const int BufLen);
;
; Description:
;		Will encrypt the buffer passed in. Does an in-place encryption on the
;     buffer so the contents will be changed. This is a very, very basic
;		encryption routine.
;
;		Will check to see if the file exists. If it does, opens the file for
;     both reading and writing and encrypts the file contents. This is to
;     allow a file once deleted to be unrecognizable by examing the hard-disk
;     or the file contents before deletion.
;
; Returns:
;     0 = success
;     1 = error opening file
;     2 = file is read-only, unable to write
;     3 = failed to write encrypted data
; ----------------------------------------------------------------------------
align 16
EncryptBuffer proc private uses ebx Buffer:dword, BufLen:dword
	; get our parameters into variables to work with
	mov ebx, [Buffer]
	mov ecx, [BufLen]
	
	; make sure it isn't a NULL pointer
	test ebx, ebx
	je Done
	
	; see if we have a zero length to process
	test ecx, ecx
	je Done
	
	; set up the crypt value
	mov edx, 0faebdc56h
	
; keep processing a byte at a time until the length left is a multiple of four
align 4
@@:
	test ecx, 3
	je @F
	mov al, byte ptr [ebx]
	add al, dl
	mov byte ptr [ebx], al
	add ebx, 1
	sub ecx, 1
	jmp @B

; now process the remaining bytes of the buffer in segments of four-bytes at a time
align 4	
@@:
	mov eax, [ebx]
	xor eax, edx
	add edx, eax
	mov [ebx], eax
	add ebx, 4
	sub ecx, 4
	jnz @B
	
Done:
	ret
EncryptBuffer endp


; ----------------------------------------------------------------------------
; prototype:
;		int __stdcall MungeFile(const char* FileSpec);
;
; Description:
;		Will check to see if the file exists. If it does, opens the file for
;     both reading and writing and encrypts the file contents. This is to
;     allow a file once deleted to be unrecognizable by examing the hard-disk
;     or the file contents before deletion.
;
; Returns:
;     0 = success
;     1 = error opening file
;     2 = file is read-only, unable to write
;     3 = failed to write encrypted data
; ----------------------------------------------------------------------------
align 16
MungeFile proc public uses ebx esi edi FileSpec:dword
	local	FileBuf      : dword
	local bytesRead    : dword
	local bytesWritten : dword
	
	; EBX = buffer (reading and writing)
	; ESI = read FileHandle
	; EDI = write FileHandle
	xor ebx, ebx
	
	; first thing we'll do is check to make sure the FileSpec is a valid pointer
	; (by seeing if it isn't NULL and the length is greater then zero)
	mov edx, [FileSpec]
	
	; test for NULL pointer
	test edx, edx
	je Done
	
	; test for zero length
	cmp dl, 0
	je Done
	
	; clear out our handles
	mov esi, INVALID_HANDLE_VALUE
	mov edi, INVALID_HANDLE_VALUE
	
	; we are going to open the file twice (once for reading only and once for
	; writing only)
	mov eax, GENERIC_READ or GENERIC_WRITE
	mov ecx, FILE_ATTRIBUTE_NORMAL or FILE_FLAG_SEQUENTIAL_SCAN
	invoke CreateFile, edx, eax, FILE_SHARE_READ or FILE_SHARE_WRITE, 0, OPEN_EXISTING, ecx, 0
	mov esi, eax
	
	mov eax, GENERIC_READ or GENERIC_WRITE
	mov ecx, FILE_ATTRIBUTE_NORMAL or FILE_FLAG_SEQUENTIAL_SCAN
	mov edx, [FileSpec]
	invoke CreateFile, edx, eax, FILE_SHARE_READ or FILE_SHARE_WRITE, 0, OPEN_EXISTING, ecx, 0
	mov edi, eax
	
	; make sure we opened the both handles
	cmp esi, INVALID_HANDLE_VALUE
	je Close
	
	cmp edi, INVALID_HANDLE_VALUE
	je Close
	
	; now that the write file is opened, set the file pointer to the beginning
	; of the file to correctly write out the encrypted data
	invoke SetFilePointer, edi, 0, 0, FILE_BEGIN
	
	; create a buffer of 16k to read segments of the file into.
	invoke GetProcessHeap
	invoke HeapAlloc, eax, 0, BUFFER_SIZE
	test eax, eax
	mov ebx, eax
	je Close

	; at this point, we have a buffer for reading and writing to our files.
	; we have two opened file handles (reading only and writing only).
align 4
@@:
	lea eax, [bytesRead]
	mov ecx, BUFFER_SIZE
	invoke ReadFile, esi, ebx, ecx, eax, 0
	cmp eax, 0
	jle Close	; error condition

	; find out how much we read
	mov ecx, [bytesRead]
	cmp ecx, 0
	je Close
	
	; encrypt the bytes
	invoke EncryptBuffer, ebx, ecx
	
	; write out the encrypted data to the file.
	lea eax, [bytesWritten]
	mov ecx, [bytesRead]
	invoke WriteFile, edi, ebx, ecx, eax, 0

	; continue with loop until we are done reading/writing	
	jmp @B

Close:
	; if the read file handle is opened
	cmp esi, INVALID_HANDLE_VALUE
	je @F
	invoke CloseHandle, esi
	
@@:
	; if the write file handle is opened
	cmp edi, INVALID_HANDLE_VALUE
	je @F
	invoke CloseHandle, edi

@@:
	; is our memory file buffer NULL?
	test ebx, ebx
	je Done
	
	; free the memory buffer
	invoke GetProcessHeap
	invoke HeapFree, eax, 0, ebx
	
Done:
	ret
MungeFile endp


; ----------------------------------------------------------------------------
; EXE entry point
; ============================================================================
align 16
EntryPoint:
	; get our EXE instance (module) handle
	invoke GetModuleHandle, NULL
	mov hInstance, eax
        
   ; munge the specified file
   invoke MungeFile, addr TestFileSpecName
   
   ; once we reach this point, the dialog has been closed, so exit the EXE
   invoke ExitProcess, eax
   
END EntryPoint
